unit Unit1;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  Let's solve the next system:                                              //
//                                                                            //
//  dx0/dt = 2t                                                               //
//  dx1/dt = 3(x0 + t^2)/2                                                    //
//  dx2/dt = 4(x1 + t*x0 + t^3)/3                                             //
//  dx3/dt = 5(x2 + t*x1 + t^2*x0 + t^4)/4                                    //
//                                                                            //
//  With next initional conditions:                                           //
//                                                                            //
//  x0(1)=1                                                                   //
//  x1(1)=1                                                                   //
//  x2(1)=1                                                                   //
//  x3(1)=1                                                                   //
//                                                                            //
//  Answer is:                                                                //
//                                                                            //
//  x0(t)=t^2                                                                 //
//  x1(t)=t^3                                                                 //
//  x2(t)=t^4                                                                 //
//  x3(t)=t^5                                                                 //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

interface

uses
  SysUtils, Forms, StdCtrls, Dialogs;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    rk_But: TButton;
    procedure rk_ButClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;


implementation

uses rk_method, Windows;




{$R *.lfm}


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// t - is independent var,                                                    //
// X[0] -> x0,                                                                //
// X[1] -> x1, etc.                                                           //

procedure Syst (var t: TFloat; var X: TFloatVector;
                var RP: TFloatVector);
begin
  RP[0] := 2.0*t;
  //means dx0/dt = 2t
  RP[1] := 3.0*(X[0]+t*t)/2.0;
  //means dx1/dt = 3(x0 + t^2)/2
  RP[2] := 4.0*(X[1]+t*X[0]+t*t*t)/3.0;
  //means dx2/dt =4(x1 + t*x0 + t^3)/3
  RP[3] := 5.0*(X[2]+t*X[1]+t*t*X[0]+t*t*t*t)/4.0;
  //means dx3/dt = 5(x2 + t*x1 + t^2*x0 + t^4)/4
end;


//                                                                            //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////




procedure TForm1.rk_ButClick(Sender: TObject);
var
    I, t1, t2: Cardinal;
    tOut, InitConds: TFloatVector;
    XOuts: TFloatMatrix;
    Points: Cardinal;
    First, Last: TFloat;
    StepsFact: Cardinal;
    Count: Word;

begin

    Memo1.Clear;

    First := 1.0;
    Last  := 11.0;
    Count:= 4;
    Points:=10+1;
    StepsFact:=1000000;

    try
    SetLength(InitConds, Count);
      InitConds[0]:=1.0;  //x0(1)=1
      InitConds[1]:=1.0;  //x1(1)=1
      InitConds[2]:=1.0;  //x2(1)=1
      InitConds[3]:=1.0;  //x3(1)=1
      SetLength(tOut, Points);
      SetLength(XOuts, Count, Points);
    except
      ShowMessage('Out of memory!!!');
      exit;
    end;

    t1:= GetTickCount();
    I := rk4fixed( Syst, InitConds, First, Last, tOut, XOuts, StepsFact );
    t2:= GetTickCount();
    Memo1.Lines.Add('Error Code = ' + IntToStr(I));
    Memo1.Lines.Add('Run time, ms = ' + IntToStr(t2-t1));
    Memo1.Lines.Add('');
    if I = 0 then
    for I := 0 to Points - 1 do
        Memo1.Lines.Add(
                   't = '+ FloatToStr(TOut[I])+
        '           x0 = '+ FloatToStr(XOuts[0, I])+
        '           x1 = '+ FloatToStr(XOuts[1, I])+
        '           x2 = '+ FloatToStr(XOuts[2, I])+
        '           x3 = '+ FloatToStr(XOuts[3, I])
        );
    //Clean memory:
    XOuts := Nil; tOut := Nil; InitConds := Nil;

end;

end.
